import java.applet.*;
import java.awt.*;
import java.net.*;
import java.io.*;
import NationalDebtRes;    // Applet controls created by Resource Wizard.
import SimpleTimer;
import SimpleTimerClient;

public class EX18A extends Applet implements Runnable, SimpleTimerClient
{
    static final int DEBT_PORT = 1996;
    static final String SUB_CMD = "SUBSCRIBE";
    static final String UNSUB_CMD = "UNSUBSCRIBE";
    InetAddress host;
    Thread thread = null;
    boolean subscribed;
    NationalDebtRes resource;
    SimpleTimer timer = null;

    public void init()
    {   // Create the resource from the Resource Wizard and 
        // initialize the controls.
        resource = new NationalDebtRes(this);
        resource.CreateControls();
        resource.IDC_DEBT.setFont(new Font("Helvetica", Font.BOLD, 18));
        resource.IDC_DEBT.setText("$ 0.00");

        subscribed = false;
    }

    public void start()
    {   // Due to security, server must be running on the same 
        // system as the document.
        try
            host = InetAddress.getByName(getCodeBase().getHost());
        catch (UnknownHostException e)
            getAppletContext().showStatus("Unable to find host.");
    }

    public boolean handleEvent(Event evt)
    {
        boolean retval = false;

        if (evt.target == resource.ID_SUBSCRIBE && !subscribed)
        {   // Attempt to subscribe to server.
            getAppletContext().showStatus("Subscribing to host...");
            if (sendCommand(SUB_CMD))
                subscribed = true;
            retval = true;
        }
        else if (evt.target == resource.ID_UNSUBSCRIBE && subscribed)
        {   // Attempt to un-subscribe from server.
            getAppletContext().showStatus("Unsubscribing from host...");
            if (sendCommand(UNSUB_CMD))
                subscribed = false;
            retval = true;
        }

        return retval;
    }

    // Sends a command to the server using a datagram.
    protected boolean sendCommand(String cmd)
    {
        boolean retval = false;
        byte buf[] = new byte[cmd.length()];
        cmd.getBytes(0, cmd.length(), buf, 0);
        DatagramSocket socket = null;

        try
        {   // Kill any running timer.
            if (timer != null)
            {
                timer.stop();
                try
                    timer.join();
                catch (InterruptedException e) {}
            }
            if (thread == null)
            {   // Start up listening thread.
                thread = new Thread(this);
                thread.start();
            }
            // Create a datagram destined for the server containing the command.
            DatagramPacket packet = new DatagramPacket(buf, buf.length, host, DEBT_PORT);
            socket = new DatagramSocket();
            socket.send(packet);    // Send it.
            // Wait 5 seconds for a response.
            timer = new SimpleTimer(this, 5000);

            retval = true;
        }
        catch (SocketException se)
            getAppletContext().showStatus("Unable to communicate with host.");
        catch (IOException e)
            getAppletContext().showStatus("Error communicating with host.");

        if (socket != null)
            socket.close();
        return retval;
    }

    public void run()
    {   // Listen on DEBT_PORT for debt updates.
        DatagramSocket listen;
        try
            listen = new DatagramSocket(DEBT_PORT);
        catch (SocketException se)
        {
            getAppletContext().showStatus("Unable to open socket to host.");
            return;
        }

        String msg;

        while (true)
        {
            try
            {
                DatagramPacket recv = new DatagramPacket(new byte[128], 128);
                listen.receive(recv);
                msg = new String(recv.getData(), 0, 0, recv.getLength());
            }
            catch (IOException e)
            {
                getAppletContext().showStatus("Error communicating with host.");
                break;
            }
            if (msg.equals(UNSUB_CMD))
            {   // Received unsubscribe confirmation, drop out.
                getAppletContext().showStatus("Unsubscribe confirmation received.");
                break;
            }
            else if (msg.equals(SUB_CMD))
            {   // Received subsription confirmation, stop timer and keep listening.
                if (timer != null)
                {
                    timer.stop();
                    try
                        timer.join();
                    catch (InterruptedException e) {}
                    timer = null;
                }
                getAppletContext().showStatus("Subscription confirmation received.");
            }
            else if (msg.charAt(0) == '$')
                resource.IDC_DEBT.setText(msg);
        }
        // Kill any running timer.
        if (timer != null)
        {
           timer.stop();
           try
               timer.join();
           catch (InterruptedException e) {}
           timer = null;
        }
    
        listen.close();
        thread = null;
    }

    // Called by SimpleTimer when a timer expires.
    public synchronized void timeOut()
    {
        timer = null;
        if (subscribed)
        {   // Trying to subscribe so reset flag.
            getAppletContext().showStatus("Subscription confirmation not " +
                    "received; server may not be active.");
            subscribed = false;
        }
        else
        {   // Trying to unsubscribe so reset flag.
            getAppletContext().showStatus("Unsubscription confirmation not received.  Try again.");
            subscribed = true;
        }
    }

    public void stop()
    {   // Make sure we unsubscribe before leaving (don't bother waiting for response).
        if (subscribed)
        {
            sendCommand(UNSUB_CMD);
            subscribed = false;
        }
        if (thread != null)
        {
            thread.stop();
            thread = null;
        }
    }
}
